#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "auth.h"

void
usage ()
{
	fprintf (stderr, "usage: setkey [-k key] user\n");
	exit (1);
}

void put_keyline (FILE *f, char *user, unsigned char *deskey);

int
main (int argc, char **argv)
{
	int c;
	char *key;
	char keybuf[100];
	unsigned char deskey[DESKEYLEN];
	char *user;
	FILE *inf, *outf;
	int userlen;
	int done;
	char buf[1000];

	key = NULL;

	while ((c = getopt (argc, argv, "k:")) != EOF) {
		switch (c) {
		case 'k':
			key = optarg;
			break;
		default:
			usage ();
		}
	}

	if (optind >= argc)
		usage ();

	user = argv[optind++];

	if (optind != argc)
		usage ();

	if (key == NULL) {
		if ((key = getpass ("Password:")) == NULL)
			exit (0);
		if (strlen (key) >= sizeof keybuf - 1) {
			fprintf (stderr, "password too long\n");
			exit (1);
		}

		strcpy (keybuf, key);

		if ((key = getpass ("Again:")) == NULL)
			exit (0);

		if (strcmp (keybuf, key) != 0) {
			fprintf (stderr, "mismatch\n");
			exit (1);
		}
	}

	remove (NEW_KEYFILE);
	if ((outf = fopen (NEW_KEYFILE, "w")) == NULL) {
		fprintf (stderr, "can't create %s\n", NEW_KEYFILE);
		exit (1);
	}
	if (chmod (NEW_KEYFILE, 0600) < 0) {
		fprintf (stderr, "can't set mode for %s\n", NEW_KEYFILE);
		exit (1);
	}

	userlen = strlen (user);
	passtokey (deskey, key);

	done = 0;
	if ((inf = fopen (KEYFILE, "r")) != NULL) {
		while (fgets (buf, sizeof buf, inf) != NULL) {
			if (strncmp (buf, user, userlen) == 0
			    && isspace (buf[userlen])) {
				put_keyline (outf, user, deskey);
				done = 1;
			} else {
				fputs (buf, outf);
			}
		}
		fclose (inf);
	}

	if (!done)
		put_keyline (outf, user, deskey);

	fflush (outf);

	if (ferror (outf) || feof (outf)) {
		fprintf (stderr, "write error\n");
		remove (NEW_KEYFILE);
		exit (1);
	}

	fclose (outf);

	if (rename (NEW_KEYFILE, KEYFILE) < 0) {
		perror ("rename");
		exit (1);
	}

	return (0);
}

void
put_keyline (FILE *outf, char *user, unsigned char *deskey)
{
	int i;

	fprintf (outf, "%s ", user);
	for (i = 0; i < DESKEYLEN; i++)
		fprintf (outf, "%02x", deskey[i]);
	fprintf (outf, "\n");
}
